package javax.swing.plaf.synth;

import javax.swing.border.Border;
import java.util.List;
import javax.swing.RowSorter;
import java.awt.Component;
import javax.swing.JTable;
import sun.swing.table.DefaultTableCellHeaderRenderer;
import java.beans.PropertyChangeEvent;
import java.awt.Graphics;
import javax.swing.table.JTableHeader;
import javax.swing.plaf.UIResource;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import javax.swing.table.TableCellRenderer;
import java.beans.PropertyChangeListener;
import javax.swing.plaf.basic.BasicTableHeaderUI;

public class SynthTableHeaderUI extends BasicTableHeaderUI implements PropertyChangeListener, SynthUI
{
    private TableCellRenderer prevRenderer;
    private SynthStyle style;
    
    public SynthTableHeaderUI() {
        this.prevRenderer = null;
    }
    
    public static ComponentUI createUI(JComponent component) {
        return new SynthTableHeaderUI();
    }
    
    @Override
    protected void installDefaults() {
        this.prevRenderer = this.header.getDefaultRenderer();
        if ((this.prevRenderer instanceof UIResource)) {
            this.header.setDefaultRenderer(new SynthTableHeaderUI.HeaderRenderer());
        }
        this.updateStyle(this.header);
    }
    
    private void updateStyle(JTableHeader tableHeader) {
        SynthContext context = this.getContext(tableHeader, 1);
        SynthStyle style = this.style;
        this.style = SynthLookAndFeel.updateStyle(context, this);
        if (this.style != style && style != null) {
            this.uninstallKeyboardActions();
            this.installKeyboardActions();
        }
        context.dispose();
    }
    
    @Override
    protected void installListeners() {
        super.installListeners();
        this.header.addPropertyChangeListener(this);
    }
    
    @Override
    protected void uninstallDefaults() {
        if ((this.header.getDefaultRenderer() instanceof SynthTableHeaderUI.HeaderRenderer)) {
            this.header.setDefaultRenderer(this.prevRenderer);
        }
        SynthContext context = this.getContext(this.header, 1);
        this.style.uninstallDefaults(context);
        context.dispose();
        this.style = null;
    }
    
    @Override
    protected void uninstallListeners() {
        this.header.removePropertyChangeListener(this);
        super.uninstallListeners();
    }
    
    @Override
    public void update(Graphics graphics, JComponent component) {
        SynthContext context = this.getContext(component);
        SynthLookAndFeel.update(context, graphics);
        context.getPainter().paintTableHeaderBackground(context, graphics, 0, 0, component.getWidth(), component.getHeight());
        this.paint(context, graphics);
        context.dispose();
    }
    
    @Override
    public void paint(Graphics graphics, JComponent component) {
        SynthContext context = this.getContext(component);
        this.paint(context, graphics);
        context.dispose();
    }
    
    protected void paint(SynthContext synthContext, Graphics g) {
        super.paint(g, synthContext.getComponent());
    }
    
    @Override
    public void paintBorder(SynthContext synthContext, Graphics graphics, int n, int n2, int n3, int n4) {
        synthContext.getPainter().paintTableHeaderBorder(synthContext, graphics, n, n2, n3, n4);
    }
    
    @Override
    public SynthContext getContext(JComponent component) {
        return this.getContext(component, SynthLookAndFeel.getComponentState(component));
    }
    
    private SynthContext getContext(JComponent component, int n) {
        return SynthContext.getContext(component, this.style, n);
    }
    
    @Override
    protected void rolloverColumnUpdated(int column, int column2) {
        this.header.repaint(this.header.getHeaderRect(column));
        this.header.repaint(this.header.getHeaderRect(column2));
    }
    
    @Override
    public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
        if (SynthLookAndFeel.shouldUpdateStyle(propertyChangeEvent)) {
            this.updateStyle((JTableHeader)propertyChangeEvent.getSource());
        }
    }
    
    static int access$000(SynthTableHeaderUI synthTableHeaderUI) {
        return synthTableHeaderUI.getRolloverColumn();
    }
    
    private class HeaderRenderer extends DefaultTableCellHeaderRenderer
    {
        HeaderRenderer() {
            this.setHorizontalAlignment(10);
            this.setName("TableHeader.renderer");
        }
        
        @Override
        public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int n) {
            int n2 = n == SynthTableHeaderUI.access$000(SynthTableHeaderUI.this);
            if (isSelected || n2 || hasFocus) {
                SynthLookAndFeel.setSelectedUI((SynthLabelUI)SynthLookAndFeel.getUIOfType(this.getUI(), SynthLabelUI.class), isSelected, hasFocus, table.isEnabled(), (boolean)n2);
            }
            else {
                SynthLookAndFeel.resetSelectedUI();
            }
            RowSorter<Object> rowSorter = (table == null) ? null : table.getRowSorter();
            List<Object> list = (rowSorter == null) ? null : rowSorter.getSortKeys();
            if (list != null && list.size() > 0 && ((RowSorter.SortKey)list.get(0)).getColumn() == table.convertColumnIndexToModel(n)) {
                switch (((RowSorter.SortKey)list.get(0)).getSortOrder()) {
                    case ASCENDING: {
                        this.putClientProperty("Table.sortOrder", "ASCENDING");
                        break;
                    }
                    case DESCENDING: {
                        this.putClientProperty("Table.sortOrder", "DESCENDING");
                        break;
                    }
                    case UNSORTED: {
                        this.putClientProperty("Table.sortOrder", "UNSORTED");
                        break;
                    }
                    default: {
                        throw new AssertionError("Cannot happen");
                    }
                }
            }
            else {
                this.putClientProperty("Table.sortOrder", "UNSORTED");
            }
            super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, n);
            return this;
        }
        
        @Override
        public void setBorder(Border border) {
            if ((border instanceof SynthBorder)) {
                super.setBorder(border);
            }
        }
    }
}
